#install.packages(
require(graphics); require(grDevices);
colors_sorted = c("darkolivegreen1", "darkolivegreen2","darkolivegreen3","brown1","brown2", "brown3")


args <- commandArgs(trailingOnly = TRUE)

if (length(args) >=1) {
	datasets_arg = args[1]
} else {
	datasets_arg = "mtu173,mtu_broad"
}

if (length(args) >= 2) {
	mut_type = args[2]
} else {
	mut_type = "P"
}

if (length(args) >= 3) {
	subsets_arg = args[3]
} else {
	subsets_arg = "s3_1000"
}

if (length(args) >= 4) {
	prefix = args[4]
} else {
	if (Sys.info()["sysname"] == "Linux") {
		prefix = "/home/misias/gwamar/"
	} else {
		prefix = "D:/Dropbox/gwamar/"
	}
}

if (length(args) >= 5) {
	scores_arg = args[5]
} else {
	scores_arg = "mi,or,lh,ws,r-tgh"
}

scores_sel = strsplit(scores_arg, "[,]")[[1]]
#scores_arg = gsub("+", ".", scores_arg, fixed=T)

if (length(args) >= 6) {
	stats_arg = args[6]
} else {
	stats_arg = "auc"
}

scoreNameAndColor <- function(score) {
	score_full = score
	if (score == "mi") {
		score_full = "MI";
		score_color = "darkolivegreen1"
	} else if (score == "or") {
		score_full = "OR";
		score_color = "darkolivegreen2"
	} else if (score == "lh") {
		score_full = "H";
		score_color = "darkolivegreen3"
	} else if(score == "mi+or+lh") {
		score_full = "RBM(MI,OR,H)"
		score_color = "darkolivegreen4"
	} else if (score %in% c("r-tgh","r.tgh")) {
		score_full = "TGH"
		score_color = "cyan"
	} else if (score == "ws"){
		score_full = "WS"
		score_color = "cyan1"
 	} else if(score == "ws+r-tgh") {
		score_full = "RBM(WS,TGH)"
		score_color = "cyan3"
	} else if(grepl("+", score_full, fixed=T)) {
		score_full = "RBM(ALL)"
		score_color = "cyan4"
	} else if(score == "TS") {
		score_full = "tree score"
		score_color = "cyan4"
	} else if (score %in% c("r-tgh_yPHYML","r.tgh_yPHYML")) {
		score_full = "TGH"
		score_color = "cyan4"
	} else if (score == "ws_yPHYML") {
		score_full = "WS"
		score_color = "cyan4"
	}
	return (list(score_full, score_color));
}

datasets = strsplit(datasets_arg, "[,]")[[1]]
subsets = strsplit(subsets_arg, "[,]")[[1]]
stats = strsplit(stats_arg, "[,]")[[1]]

rows_count = length(subsets)
cols_count = length(datasets)

outdir = paste(prefix, "figures/", sep="")
dir.create(outdir, showWarnings = FALSE)

time_str = format(Sys.time(), "%Y-%m-%d_%H-%M-%S")

output_fn = paste(outdir, "/auc_bars_", time_str, ".pdf", sep="")
pdf(output_fn, width=18, height=7*length(stats)*length(subsets))
par(mfrow=c(rows_count,cols_count), las=2, mar=c(11,6,2,1))

for (stat in stats) {
	for (subset in subsets) {
		for (dataset in datasets) {

			input_fn = paste(prefix, "datasets/", dataset, "/ex", mut_type, "/comparison/stats/", subset, "/",stat,".txt", sep="")

			if (file.exists(input_fn)){
				data = read.table(input_fn, header=TRUE, sep="\t")

				input_data_pn = paste(prefix,"datasets/", dataset, "/ex",mut_type,"/comparison/stats/", subset, "/stats_pn.txt", sep="")
				data_pn = read.table(input_data_pn, header=TRUE, sep="\t")
				positives = data_pn[,"ps"]
				negatives = data_pn[,"ns"]

				#main_txt = paste("AUC barplots; ", subset, ",",dataset, " dataset; positives: ", toString(positives), "; negatives: " , toString(negatives), sep="")
				main_txt = paste("Barplots for AUC; ",dataset, "; positives: ", toString(positives), "; negatives: " , toString(negatives), sep="")

				cnames = colnames(data)

				cnames_int_full = c()
				cnames_int = c()
				score_colors = c()

				for (cname_tmp in scores_sel) {
					score_ret = scoreNameAndColor(cname_tmp)
					print(score_ret)
					score_name = score_ret[[1]]
					score_color = score_ret[[2]]
					cname = gsub("+", ".", cname_tmp, fixed=T)
					cname = gsub("-", ".", cname, fixed=T)
					if (cname %in% cnames) {
						cnames_int = c(cnames_int, cname)
						cnames_int_full = c(cnames_int_full, score_name)
						score_colors = c(score_colors, score_color)
					}
				}
				print("score_colors")
				print(score_colors)
				print("scores_arg")
				print(scores_arg)
				print("scores_sel")
				print(scores_sel)
				print("cnames")
				print(cnames)
				print("cnames_int")
				print(cnames_int)
				print("cnames_int_full")
				print(cnames_int_full)

				end_point = 0.5 + 2*length(cnames_int_full)-1 
			 	boxplot(data[, cnames_int], 
					names=cnames_int_full,
					main=main_txt, 
					col=score_colors,
					cex.axis=2.5,
					cex.main=2.0,
					xaxt="n")
			 		y_pos = par("usr")[3]-0.05*(par("usr")[4]-par("usr")[3])
			 		text(cex=2.3, x=seq(1.0, length(cnames_int_full)*1.05, 1.05), y=y_pos, labels=paste(cnames_int_full), adj= 1, xpd = TRUE, srt=45, pos=2)
			 } else {
			 	plot(0.0,0.0, 
			 		col="white", 
			 		cex.main=1.6,
			 		cex.lab=2.2, cex.axis=2.2,
			 		main="empty plot...")
			 }
		}
	}
}
dev.off()
