#install.packages(
require(graphics); require(grDevices);
library(vioplot)
#data_fn = paste(species_id, "_out/r_subset/", drug_name, ".txt", sep = "")
colors_sorted = c("lightgreen", "lightblue","peachpuff","azure2","red")

library(sm)

my.vioplot <- function(x,...,range=1.5,h=NULL,ylim=NULL,names=NULL, horizontal=FALSE,
  col="magenta", border="black", lty=1, lwd=1, rectCol="black", colMed="white", pchMed=19, at, add=FALSE, wex=1,
  drawRect=TRUE)
{
    # process multiple datas
    datas <- list(x,...)
    n <- length(datas)

    if(missing(at)) at <- 1:n

    # pass 1
    #
    # - calculate base range
    # - estimate density
    #

    # setup parameters for density estimation
    upper  <- vector(mode="numeric",length=n)
    lower  <- vector(mode="numeric",length=n)
    q1     <- vector(mode="numeric",length=n)
    q3     <- vector(mode="numeric",length=n)
    med    <- vector(mode="numeric",length=n)
    base   <- vector(mode="list",length=n)
    height <- vector(mode="list",length=n)
    baserange <- c(Inf,-Inf)

    # global args for sm.density function-call
    args <- list(display="none")

    if (!(is.null(h)))
        args <- c(args, h=h)

    for(i in 1:n) {
        data<-datas[[i]]

        # calculate plot parameters
        #   1- and 3-quantile, median, IQR, upper- and lower-adjacent
        data.min <- min(data)
        data.max <- max(data)
        q1[i]<-quantile(data,0.25)
        q3[i]<-quantile(data,0.75)
        med[i]<-median(data)
        iqd <- q3[i]-q1[i]
        upper[i] <- min( q3[i] + range*iqd, data.max )
        lower[i] <- max( q1[i] - range*iqd, data.min )

        #   strategy:
        #       xmin = min(lower, data.min))
        #       ymax = max(upper, data.max))
        #

        est.xlim <- c( min(lower[i], data.min), max(upper[i], data.max) )

        # estimate density curve
        smout <- do.call("sm.density", c( list(data, xlim=est.xlim), args ) )

        # calculate stretch factor
        #
        #  the plots density heights is defined in range 0.0 ... 0.5
        #  we scale maximum estimated point to 0.4 per data
        #
        hscale <- 0.4/max(smout$estimate) * wex

        # add density curve x,y pair to lists
        base[[i]]   <- smout$eval.points
        height[[i]] <- smout$estimate * hscale

        # calculate min,max base ranges
        t <- range(base[[i]])
        baserange[1] <- min(baserange[1],t[1])
        baserange[2] <- max(baserange[2],t[2])

    }

    # pass 2
    #
    # - plot graphics

    # setup parameters for plot
    if(!add){
      xlim <- if(n==1)
               at + c(-.5, .5)
              else
               range(at) + min(diff(at))/2 * c(-1,1)

      if (is.null(ylim)) {
         ylim <- baserange
      }
    }
    if (is.null(names)) {
        label <- 1:n
    } else {
        label <- names
    }

    boxwidth <- 0.05 * wex

    # setup plot
    if(!add)
      plot.new()
    if(!horizontal) {
      if(!add){
        plot.window(xlim = xlim, ylim = ylim)
        axis(2)
        axis(1,at = at, label=label )
      }

      box()
      for(i in 1:n) {
          # plot left/right density curve
          polygon( c(at[i]-height[[i]], rev(at[i]+height[[i]])),
                   c(base[[i]], rev(base[[i]])),
                   col = col[i %% length(col) + 1], border=border, lty=lty, lwd=lwd)

          if(drawRect){
            # plot IQR
            lines( at[c( i, i)], c(lower[i], upper[i]) ,lwd=lwd, lty=lty)

            # plot 50% KI box
            rect( at[i]-boxwidth/2, q1[i], at[i]+boxwidth/2, q3[i], col=rectCol)

            # plot median point
            points( at[i], med[i], pch=pchMed, col=colMed )
         }
      }

    }
    else {
      if(!add){
        plot.window(xlim = ylim, ylim = xlim)
        axis(1)
        axis(2,at = at, label=label )
      }

      box()
      for(i in 1:n) {
          # plot left/right density curve
          polygon( c(base[[i]], rev(base[[i]])),
                   c(at[i]-height[[i]], rev(at[i]+height[[i]])),
                   col = col[i %% length(col) + 1], border=border, lty=lty, lwd=lwd)

          if(drawRect){
            # plot IQR
            lines( c(lower[i], upper[i]), at[c(i,i)] ,lwd=lwd, lty=lty)

            # plot 50% KI box
            rect( q1[i], at[i]-boxwidth/2, q3[i], at[i]+boxwidth/2,  col=rectCol)

            # plot median point
            points( med[i], at[i], pch=pchMed, col=colMed )
          }
      }
    }
    invisible (list( upper=upper, lower=lower, median=med, q1=q1, q3=q3))
}

args <- commandArgs(trailingOnly = TRUE)

if (length(args) >=1) {
	dataset = args[1]
} else {
	#dataset="sau461"
	#dataset = "mtu173"
	dataset = "mtu_broad"
}

if (length(args) >= 2) {
	mut_type = args[2]
} else {
	mut_type = "P"
}

if (length(args) >= 3) {
	subset = args[3]
} else {
	subset = "sel3"
}

if (length(args) >= 4) {
	prefix = args[4]
} else {
	if (Sys.info()["sysname"] == "Linux") {
		prefix = "/home/misias/gwamar/"
	} else {
		prefix = "D:/Dropbox/Dropbox/gwamar/"
	}
}

if (length(args) >= 5) {
	scores_arg = args[5]
} else {
	scores_arg = "mi,or,lh,ws_yPHYML,r.tgh_yPHYML"
}

scoreFullName <- function(score) {
	score_full = score
	if (score == "mi") {
		score_full = "mutual information";
	} else if (score == "or") {
		score_full = "odds ratio";
	} else if (score == "lh") {
		score_full = "hypergeometric";
	} else if (score %in% c("r-tgh","r.tgh")) {
		score_full = "TGH"
	} else if (score == "ws"){
		score_full = "weighted support"
	} else if(score == "ws+lh+r-tgh+mi+or") {
		score_full = "combined"
	} else if(score == "ts") {
		score_full = "tree score"
	} else if (score %in% c("r-tgh_yPHYML","r.tgh_yPHYML")) {
		score_full = "TGH (PHYML)"
	} else if (score == "ws_yPHYML") {
		score_full = "weighted support (PHYML)"
	} else if (score == "ws_yRAXML") {
		score_full = "weighted support (RAXML)"
	} else if (score == "r-tgh_yRAXML") {
		score_full = "TGH (RAXML)"
	}
	return(score_full);
}

scoreFullNameShort <- function(score) {
	score_full = score
	if (score == "mi") {
		score_full = "MI";
	} else if (score == "or") {
		score_full = "OR";
	} else if (score == "lh") {
		score_full = "H";
	} else if (score %in% c("r-tgh","r.tgh")) {
		score_full = "TGH"
	} else if (score == "ws"){
		score_full = "WS"
	} else if(score == "ws+lh+r-tgh+mi+or") {
		score_full = "combined"
	} else if(score == "TS") {
		score_full = "tree score"
	} else if (score %in% c("r-tgh_yPHYML","r.tgh_yPHYML")) {
		score_full = "TGH"
	} else if (score == "ws_yPHYML") {
		score_full = "WS"
	} else if (score == "ws_yRAXML") {
		score_full = "WS (RAXML)"
	} else if (score == "r-tgh_yRAXML") {
		score_full = "TGH (RAXML)"
	}
	return(score_full);
}



outdir = paste(prefix,"datasets/paper/", sep="")
dir.create(outdir, showWarnings = FALSE)

output_fn = paste(outdir, "/auc_vio_", subset, ".pdf", sep="")

pdf(output_fn, width=18,height=10)
par(mfrow=c(1,2), las=2, mar=c(7,6,2,1))


for (dataset in c("mtu173", "mtu_broad")) {

  input_fn = paste(prefix, "datasets/", dataset, "/ex", mut_type, "/comparison/stats/", subset, ".txt", sep="")
  data = read.table(input_fn, header=TRUE, sep="\t")

  input_data_pn = paste(prefix,"datasets/",dataset,"/ex",mut_type,"/comparison/stats/",subset,"/stats_pn.txt", sep="")
  data_pn = read.table(input_data_pn, header=TRUE, sep="\t")
  positives = data_pn[,"ps"]
  negatives = data_pn[,"ns"]


  main_txt = paste("AUC barplots; ", dataset, " dataset; positives: ", toString(positives), "; negatives: " , toString(negatives), sep="")

  print(scores_arg)
  scores_sel = strsplit(scores_arg, "[,]")[[1]]
  print("scores_sel")
  print(scores_sel)
  cnames = colnames(data)
  print("cnames")
  print(cnames)

  cnames_int_full = c()
  cnames_int = c()

  for (cname in scores_sel) {
    if (cname %in% cnames) {
      cnames_int = c(cnames_int, cname)
      cnames_int_full = c(cnames_int_full, scoreFullNameShort(cname))
    }
  }

  end_point = 0.5 + 2*length(cnames_int_full)-1 
	x1 = data[, cnames_int[[1]]]
	x2 = data[, cnames_int[[2]]]
	x3 = data[, cnames_int[[3]]]
	x4 = data[, cnames_int[[4]]]
	x5 = data[, cnames_int[[5]]]

	my.vioplot(x1,x2,x3,x4,x5,
		col=colors_sorted,
		names=cnames_int_full
			)
}

dev.off()
