#install.packages(
require(graphics); require(grDevices);

#data_fn = paste(species_id, "_out/r_subset/", drug_name, ".txt", sep = "")

args <- commandArgs(trailingOnly = TRUE)

if (length(args) >=1) {
	dataset = args[1]
} else {
	#dataset="sau461"
	#dataset = "mtu173"
	dataset = "mtu_broad"
}

if (length(args) >= 2) {
	mut_type = args[2]
} else {
	mut_type = "P"
}

if (length(args) >= 3) {
	subset = args[3]
} else {
	subset = "sel3"
}

if (length(args) >= 4) {
	prefix = args[4]
} else {
	if (Sys.info()["sysname"] == "Linux") {
		prefix = "/home/misias/Dropbox/gwamar/"
	} else {
		prefix = "D:/Dropbox/Dropbox/gwamar/"
	}
}

if (length(args) >= 5) {
	scores_arg = args[5]
} else {
	scores_arg = "mi,or,r-tgh,ws,lh"
}

scoreFullName <- function(score) {
	score_full = score
	if (score == "mi") {
		score_full = "mutual information";
	} else if (score == "or") {
		score_full = "odds ratio";
	} else if (score == "lh") {
		score_full = "hypergeometric";
	} else if (score %in% c("r-tgh","r.tgh")) {
		score_full = "TGH"
	} else if (score == "ws"){
		score_full = "weighted support"
	} else if(score == "ws+lh+r-tgh+mi+or") {
		score_full = "combined"
	} else if(score == "ts") {
		score_full = "tree score"
	} else if (score %in% c("r-tgh_yPHYML","r.tgh_yPHYML")) {
		score_full = "TGH (PHYML)"
	} else if (score == "ws_yPHYML") {
		score_full = "weighted support (PHYML)"
	} else if (score == "ws_yRAXML") {
		score_full = "weighted support (RAXML)"
	} else if (score == "r-tgh_yRAXML") {
		score_full = "TGH (RAXML)"
	}
	return(score_full);
}


input_fn = paste(prefix,"datasets/",dataset,"/ex",mut_type,"/comparison/", subset, ".txt", sep="")
print(input_fn)
#score_files = list.files(path = inputdir)
#print(score_files)


outdir = paste(prefix,"datasets/",dataset,"/ex",mut_type,"/figures/", sep="")
dir.create(outdir, showWarnings = FALSE)

#input_fn = paste(inputdir, "/",score_files[1], sep="")
data = read.table(input_fn, header=TRUE, sep="\t")

input_data_pn = paste(prefix,"datasets/",dataset,"/ex",mut_type,"/comparison/stats_pn_", subset, ".txt", sep="")
data_pn = read.table(input_data_pn, header=TRUE, sep="\t")
positives = data_pn[,"ps"]
negatives = data_pn[,"ns"]


main_txt = paste("AUC barplots; positives: ", toString(positives), "; negatives: " , toString(negatives), sep="")

print(scores_arg)
scores_sel = strsplit(scores_arg, "[,]")[[1]]
print("scores_sel")
print(scores_sel)
cnames = colnames(data)
print("cnames")
print(cnames)

cnames_int_full = c()
cnames_int = c()

for (cname in scores_sel) {
	if (cname %in% cnames) {
		cnames_int = c(cnames_int, cname)
		cnames_int_full = c(cnames_int_full, scoreFullName(cname))
	}
}

output_fn = paste(outdir, "/pr_bars_",dataset, "_", subset, ".pdf", sep="")
pdf(output_fn, width=18,height=10)
boxplot(data[, cnames_int], names=cnames_int_full,  main=main_txt, par=2)
#boxplot(data[,], par=2)
dev.off()
